@@ -59,6 +59,8 @@ struct BuilderOptions
   char      **build_args;
   char      **test_args;
   char      **config_opts;
+  char      **secret_opts;
+  char      **secret_env;
   char      **make_args;
   char      **make_install_args;
   GHashTable *arch;
@@ -94,6 +96,8 @@ enum {
   PROP_BUILD_ARGS,
   PROP_TEST_ARGS,
   PROP_CONFIG_OPTS,
+  PROP_SECRET_OPTS,
+  PROP_SECRET_ENV,
   PROP_MAKE_ARGS,
   PROP_MAKE_INSTALL_ARGS,
   PROP_APPEND_PATH,
@@ -127,6 +131,8 @@ builder_options_finalize (GObject *object)
   g_strfreev (self->build_args);
   g_strfreev (self->test_args);
   g_strfreev (self->config_opts);
+  g_strfreev (self->secret_opts);
+  g_strfreev (self->secret_env);
   g_strfreev (self->make_args);
   g_strfreev (self->make_install_args);
   g_hash_table_destroy (self->arch);
@@ -228,6 +234,14 @@ builder_options_get_property (GObject    *object,
       g_value_set_boxed (value, self->config_opts);
       break;
 
+    case PROP_SECRET_OPTS:
+      g_value_set_boxed (value, self->secret_opts);
+      break;
+
+    case PROP_SECRET_ENV:
+      g_value_set_boxed (value, self->secret_env);
+      break;
+
     case PROP_MAKE_ARGS:
       g_value_set_boxed (value, self->make_args);
       break;
@@ -370,6 +384,18 @@ builder_options_set_property (GObject      *object,
       g_strfreev (tmp);
       break;
 
+    case PROP_SECRET_OPTS:
+      tmp = self->secret_opts;
+      self->secret_opts = g_strdupv (g_value_get_boxed (value));
+      g_strfreev (tmp);
+      break;
+
+    case PROP_SECRET_ENV:
+      tmp = self->secret_env;
+      self->secret_env = g_strdupv (g_value_get_boxed (value));
+      g_strfreev (tmp);
+      break;
+
     case PROP_MAKE_ARGS:
       tmp = self->make_args;
       self->make_args = g_strdupv (g_value_get_boxed (value));
@@ -555,6 +581,23 @@ builder_options_class_init (BuilderOptionsClass *klass)
                                                        "",
                                                        G_TYPE_STRV,
                                                        G_PARAM_READWRITE));
+
+  g_object_class_install_property (object_class,
+                                   PROP_SECRET_OPTS,
+                                   g_param_spec_boxed ("secret-opts",
+                                                       "",
+                                                       "",
+                                                       G_TYPE_STRV,
+                                                       G_PARAM_READWRITE));
+
+  g_object_class_install_property (object_class,
+                                   PROP_SECRET_ENV,
+                                   g_param_spec_boxed ("secret-env",
+                                                       "",
+                                                       "",
+                                                       G_TYPE_STRV,
+                                                       G_PARAM_READWRITE));
+
   g_object_class_install_property (object_class,
                                    PROP_MAKE_ARGS,
                                    g_param_spec_boxed ("make-args",
@@ -815,7 +858,7 @@ get_all_options (BuilderOptions *self, BuilderContext *context)
   return options;
 }
 
-static const char *
+static char *
 builder_options_get_flags (BuilderOptions *self,
                            BuilderContext *context,
                            size_t          field_offset,
@@ -870,7 +913,7 @@ get_sdk_flags (BuilderOptions *self, BuilderContext *context, const char *(*meth
   return NULL;
 }
 
-const char *
+char *
 builder_options_get_cflags (BuilderOptions *self, BuilderContext *context)
 {
   return builder_options_get_flags (self, context, G_STRUCT_OFFSET (BuilderOptions, cflags),
@@ -878,7 +921,7 @@ builder_options_get_cflags (BuilderOptions *self, BuilderContext *context)
                                     get_sdk_flags (self, context, builder_sdk_config_get_cflags));
 }
 
-const char *
+char *
 builder_options_get_cxxflags (BuilderOptions *self, BuilderContext *context)
 {
   return builder_options_get_flags (self, context, G_STRUCT_OFFSET (BuilderOptions, cxxflags),
@@ -886,7 +929,7 @@ builder_options_get_cxxflags (BuilderOptions *self, BuilderContext *context)
                                     get_sdk_flags (self, context, builder_sdk_config_get_cxxflags));
 }
 
-const char *
+char *
 builder_options_get_cppflags (BuilderOptions *self, BuilderContext *context)
 {
   return builder_options_get_flags (self, context, G_STRUCT_OFFSET (BuilderOptions, cppflags),
@@ -894,7 +937,7 @@ builder_options_get_cppflags (BuilderOptions *self, BuilderContext *context)
                                     get_sdk_flags (self, context, builder_sdk_config_get_cppflags));
 }
 
-const char *
+char *
 builder_options_get_ldflags (BuilderOptions *self, BuilderContext *context)
 {
   return builder_options_get_flags (self, context, G_STRUCT_OFFSET (BuilderOptions, ldflags),
@@ -1091,7 +1134,10 @@ builder_options_get_env (BuilderOptions *self, BuilderContext *context)
   GList *l;
   int i;
   char **envp = NULL;
-  const char *cflags, *cppflags, *cxxflags, *ldflags;
+  g_autofree char *cflags = NULL;
+  g_autofree char *cppflags = NULL;
+  g_autofree char *cxxflags = NULL;
+  g_autofree char *ldflags = NULL;
 
   envp = builder_context_extend_env_pre (context, envp);
 
@@ -1274,6 +1320,22 @@ builder_options_get_config_opts (BuilderOptions *self,
   return builder_options_get_strv (self, context, base_opts, G_STRUCT_OFFSET (BuilderOptions, config_opts));
 }
 
+char **
+builder_options_get_secret_opts (BuilderOptions *self,
+                                 BuilderContext *context,
+                                 char          **base_opts)
+{
+  return builder_options_get_strv (self, context, base_opts, G_STRUCT_OFFSET (BuilderOptions, secret_opts));
+}
+
+char **
+builder_options_get_secret_env (BuilderOptions *self,
+                                BuilderContext *context,
+                                char          **base_opts)
+{
+  return builder_options_get_strv (self, context, base_opts, G_STRUCT_OFFSET (BuilderOptions, secret_env));
+}
+
 char **
 builder_options_get_make_args (BuilderOptions *self,
                                BuilderContext *context,
@@ -1312,6 +1374,8 @@ builder_options_checksum (BuilderOptions *self,
   builder_cache_checksum_strv (cache, self->build_args);
   builder_cache_checksum_compat_strv (cache, self->test_args);
   builder_cache_checksum_strv (cache, self->config_opts);
+  builder_cache_checksum_strv (cache, self->secret_opts);
+  builder_cache_checksum_strv (cache, self->secret_env);
   builder_cache_checksum_strv (cache, self->make_args);
   builder_cache_checksum_strv (cache, self->make_install_args);
   builder_cache_checksum_boolean (cache, self->strip);
