@@ -45,6 +45,7 @@ struct BuilderSourceFile
   char         *sha512;
   char         *dest_filename;
   char         *http_referer;
+  gboolean      disable_http_decompression;
 };
 
 typedef struct
@@ -65,6 +66,7 @@ enum {
   PROP_DEST_FILENAME,
   PROP_MIRROR_URLS,
   PROP_HTTP_REFERER,
+  PROP_DISABLE_HTTP_DECOMPRESSION,
   LAST_PROP
 };
 
@@ -132,6 +134,10 @@ builder_source_file_get_property (GObject    *object,
       g_value_set_string (value, self->http_referer);
       break;
 
+    case PROP_DISABLE_HTTP_DECOMPRESSION:
+      g_value_set_boolean (value, self->disable_http_decompression);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -206,6 +212,10 @@ builder_source_file_set_property (GObject      *object,
       self->http_referer = g_value_dup_string (value);
       break;
 
+    case PROP_DISABLE_HTTP_DECOMPRESSION:
+      self->disable_http_decompression = g_value_get_boolean (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -335,40 +345,62 @@ get_source_file (BuilderSourceFile *self,
   return NULL;
 }
 
+#define BASE64_INDICATOR ";base64"
+#define BASE64_INDICATOR_LEN (strlen (BASE64_INDICATOR))
+
 static GBytes *
 download_data_uri (const char     *url,
-                   BuilderContext *context,
                    GError        **error)
 {
-  CURL *session;
-  g_autoptr(GError) first_error = NULL;  
-  g_autoptr(GInputStream) input = NULL;
-  g_autoptr(GOutputStream) out = NULL;
-  g_autoptr(GUri) parsed = NULL;
+  gboolean is_base64 = FALSE;
+  const char *comma, *start;
+  g_autoptr(GBytes) bytes = NULL;
 
-  parsed = g_uri_parse(url, CONTEXT_HTTP_URI_FLAGS, error);
-  if (!parsed)
+  /* This is a simplified version of soup_uri_decode_data_uri()
+   * SPDX-License-Identifier: LGPL-2.0-or-later
+   */
+
+  start = url + strlen ("data:");
+  comma = strchr (start, ',');
+
+  /* Check params to see if it is base64. */
+  if (comma && comma != start)
+  {
+    if (comma >= start + BASE64_INDICATOR_LEN && !g_ascii_strncasecmp (comma - BASE64_INDICATOR_LEN,
+                                                                        BASE64_INDICATOR, BASE64_INDICATOR_LEN))
+      is_base64 = TRUE;
+  }
+
+  if (comma)
+    start = comma + 1;
+
+  if (*start)
     {
-      return FALSE;
+      bytes = g_uri_unescape_bytes (start, -1, NULL, error);
+      if (!bytes)
+        return NULL;
+
+      if (is_base64)
+        {
+          GByteArray *unescaped_array;
+          gsize content_length;
+
+          if (g_bytes_get_size (bytes) <= 1)
+            {
+              g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED, "data URI contains invalid base64 data");
+              return NULL;
+            }
+
+          unescaped_array = g_bytes_unref_to_array (bytes);
+          g_base64_decode_inplace ((char*)unescaped_array->data, &content_length);
+          unescaped_array->len = content_length;
+          bytes = g_byte_array_free_to_bytes (unescaped_array);
+        }
     }
+  else
+    bytes = g_bytes_new_static (NULL, 0); /* Empty data */
 
-  session = builder_context_get_curl_session (context);
-  out = g_memory_output_stream_new_resizable ();
-
-  if (!builder_download_uri_buffer (parsed, NULL, session, out, NULL, 0, error))
-    {
-      g_propagate_error (error, g_steal_pointer (&first_error));
-      return NULL;
-    }
-
-  if (!g_output_stream_splice (out,
-                               input,
-                               G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET | G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE,
-                               NULL,
-                               error))
-    return NULL;
-
-  return g_memory_output_stream_steal_as_bytes (G_MEMORY_OUTPUT_STREAM (out));
+  return g_steal_pointer (&bytes);
 }
 
 static gboolean
@@ -436,6 +468,7 @@ builder_source_file_download (BuilderSource  *source,
                                      self->url,
                                      (const char **)self->mirror_urls,
                                      self->http_referer,
+                                     self->disable_http_decompression,
                                      file,
                                      checksums,
                                      checksums_type,
@@ -493,7 +526,6 @@ builder_source_file_extract (BuilderSource  *source,
       g_autoptr(GBytes) content = NULL;
 
       content = download_data_uri (self->url,
-                                   context,
                                    error);
       if (content == NULL)
         return FALSE;
@@ -707,6 +739,14 @@ builder_source_file_class_init (BuilderSourceFileClass *klass)
                                                         "",
                                                         NULL,
                                                         G_PARAM_READWRITE));
+
+  g_object_class_install_property (object_class,
+                                   PROP_DISABLE_HTTP_DECOMPRESSION,
+                                   g_param_spec_boolean ("disable-http-decompression",
+                                                        "",
+                                                        "",
+                                                        FALSE,
+                                                        G_PARAM_READWRITE));
 }
 
 static void
