@@ -23,7 +23,7 @@ set -euo pipefail
 
 skip_without_fuse
 
-echo "1..8"
+echo "1..10"
 
 setup_repo
 install_repo
@@ -53,6 +53,8 @@ cp $(dirname $0)/org.flatpak_builder.gui.desktop .
 cp $(dirname $0)/org.flatpak_builder.gui.json .
 cp $(dirname $0)/org.flatpak_builder.gui.metainfo.xml .
 cp $(dirname $0)/org.test.Hello.png .
+cp $(dirname $0)/org.test.Hello-256.png .
+cp $(dirname $0)/org.flatpak.appstream_media.json .
 mkdir include1
 cp $(dirname $0)/module1.json include1/
 cp $(dirname $0)/module1.yaml include1/
@@ -141,6 +143,33 @@ ${FLATPAK_BUILDER} --repo=$REPO/repo_sc --force-clean builddir_sc \
     org.flatpak_builder.gui.json >&2
 ostree --repo=$REPO/repo_sc refs|grep -Eq "^screenshots/$(flatpak --default-arch)$"
 ostree checkout --repo=$REPO/repo_sc -U screenshots/$(flatpak --default-arch) outdir_sc
-find outdir_sc -path "*/screenshots/image-1_orig.png" -type f | grep -q .
+find outdir_sc -path "*/icons/64x64/org.test.Hello.png" -type f | grep -q .
 
 echo "ok screenshot ref commit"
+
+# test compose partial url policy
+${FLATPAK_BUILDER} --force-clean builddir_sc \
+    --mirror-screenshots-url=https://example.org/media \
+    --state-dir .fp-compose-url-policy-partial \
+    --compose-url-policy=partial \
+    org.flatpak.appstream_media.json >&2
+# we test for the icon tag instead of screenshot
+# the former works offline the latter does not
+gzip -cdq builddir_sc/files/share/app-info/xmls/org.flatpak.appstream_media.xml.gz|grep -Eq '>org/flatpak/appstream_media/[^/]+/icons/128x128/org.flatpak.appstream_media.png</icon>'
+
+echo "ok compose partial url policy"
+
+# test compose full url policy
+if appstream_has_version 0 16 3; then
+    ${FLATPAK_BUILDER} --force-clean builddir_sc \
+        --mirror-screenshots-url=https://example.org/media \
+        --state-dir .fp-compose-url-policy-full \
+        --compose-url-policy=full \
+        org.flatpak.appstream_media.json >&2
+
+    gzip -cdq builddir_sc/files/share/app-info/xmls/org.flatpak.appstream_media.xml.gz|grep -Eq '>https://example.org/media/org/flatpak/appstream_media/[^/]+/icons/128x128/org.flatpak.appstream_media.png</icon>'
+
+    echo "ok compose full url policy"
+else
+    echo "ok # Skip AppStream < 0.16.3"
+fi
